<?php
/*
  $Id: shipping_modules_customization.php,v 1.1 2002/02/02 20:32:09 dgw_ Exp $

  osCommerce, Open Source E-Commerce Solutions
  http://www.oscommerce.com

  Copyright (c) 2002 osCommerce

  Released under the GNU General Public License
*/
?>
<P class=contentHeading>General</P>

<P>To write a custom shipping module, you have basically two choices: start with the simplest module , or start with the most similar module.</P>

<P>The first approach is maybe best for understanding how it works. The second might mean less development work, but you will have to spend some more time analyzing how the complex module works.</P>

<P class="stepsBox">We will start from the very simple module &#147;flat&#148;. We will write a module where the cost is a percentage of the cart total.</P>

<P>Anyway, the first step is to read both <a href="shipping_modules-installation.htm">Installation</a> and <a href="shipping_modules-structure.htm">Structure</a> pages to get a basic understanding of the shipping modules.</P>

<P class="contentHeading">Starting base</P>

<P>First we will copy the base file and rename everything so our variables and constants do not collide with those for the base module.</P>

<P class="stepsBox">Copy includes/modules/shipping/flat.php to percent.php<br>
Copy includes/modules/languages/english/modules/shipping/flat.php to percent.php<br>
(We will take care of the other languages afterwards.) </P>

<P>Now edit the program file with a good text editor (notepad.exe will do, but you will suffer. I recommend <a href="http://www.ultraedit.com/">UltraEdit</a> on Windows, <a href="http://www.vim.org/">vi</a> on Unix.</P>

<P class="stepsBox">Go through the file and replace all occurences of 'flat' with 'percent' (mind the case). You should get 37 occurences (this from version 1.30 of the file). Eg.:<br>
Line 13: 'class flat' becomes 'class percent'<br>
<br>
Now, replace 'FLAT' with 'PERCENT'. You should get 14 occurences.</P>

<P>Now edit the language file.</P>

<p class="stepsBox">Replace 'FLAT' with 'PERCENT'. You should get 3 occurences (that's from version 1.4).</p>

<P>After that, a little sanity check. Fire your browser, point to admin. Install the percent module and check it works as flat does. (We only changed the names, after all!) When testing is finished, disable the module because we are going to modify the constants. We don't want any old stuff poluting the database. You will have to do this each time you modify the file.</P>

<P>Most functions will be left untouched:</P>

<ul>
  <li><b>selection()</b></li>
  <li><b>cheapest()</b></li>
  <li><b>display()</b></li>
  <li><b>confirm()</b></li>
  <li><b>check()</b></li>
</ul>

<P>Let's start with quote(), which is the core of the module.</P>

<P class="noteBox">Note: use spaces for alignments, not tabulations.</P>

<p class=contentHeading>Modification of quote()</p>

<p>What do we need to calculate the shipping? We need to get this information either from the catalog or from the module. In the last case, we will use a constant which will be modifiable with admin.</p>

<p class="stepsBox"> In our example, we need the cart total and the rate (percent) for the shipping. The first we'll get from the catalog software. The rate is defined by the administrator: we will use a constant.</p>

<p class="stepsBox">Edit the program file. Add $cart to the globals:<br>
<br>
global $cart, $shipping_quoted, $shipping_percent_cost, $shipping_percent_method;<br>
<br>
Now get the total cost of items:<br>
<br>
$shipping_total = $cart-&gt;show_total();</p>

<p>When all data is available, we can do the calculation.</p>

<p class="stepsBox">$shipping_percent_cost = MODULE_SHIPPING_PERCENT_RATE * $shipping_total; </p>

<p>That's it!</p>

<pre class="stepsBox">Here is the whole function:<br>
    function quote() {
      global $cart, $shipping_quoted, $shipping_percent_cost, $shipping_percent_method;

      $shipping_total = $cart->show_total();
      if ( ($GLOBALS['shipping_quote_all'] == '1') || ($GLOBALS['shipping_quote_percent'] == '1') ) {
        $shipping_quoted = 'percent';
        $shipping_percent_cost = MODULE_SHIPPING_PERCENT_RATE/100 * $shipping_total;
        $shipping_percent_method = MODULE_SHIPPING_PERCENT_TEXT_WAY;
      }
    }       
</pre>

<p>Easy, isn't it? Well, the calculation can become more complex. The zones module deals with multiple zones and weights. Some modules, such as fedex, have their own class file (in includes/classes) because the function is very complex (the class will fetch the rates on the web.) This is beyond the scope of this tutorial.</p>

<p class=contentHeading>Modification of install()</p>

<p>If you needed new constants, you have to introduce them in the database. If you did not use all of the constants from the base module, get rid of them. The job is straightforward with a single query.</p>

<p class="stepsBox">Lets add MODULE_SHIPPING_PERCENT_RATE, and remove MODULE_SHIPPING_PERCENT_COST which we don't use.<br>
Modify the line with MODULE_SHIPPING_PERCENT_COST to read (leave all on one line): </p>

<pre class="stepsBox">
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, 
                                                             configuration_key,
                                                             configuration_value,
                                                             configuration_description,
                                                             configuration_group_id,
                                                             sort_order,
                                                             date_added)
                    values ('Percentage rate',
                            'MODULE_SHIPPING_PERCENT_RATE',
                            '10.00',
                            'What is the shipping rate (percent)? The Handling fee will also be added.',
                            '6',
                            '6',
                            now())");
</pre>

<p class="stepsBox">Also modify the captions in the line with MODULE_SHIPPING_PERCENT_STATUS.</p>

<p class=contentHeading>Modification of remove()</p>

<p class=content>All constants need to be removed from the database when disabling the module.</p>

<p class="stepsBox"> Modify the line with MODULE_SHIPPING_PERCENT_COST to:<br>
tep_db_query("delete from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_SHIPPING_PERCENT_RATE'"); </p>

<p class=contentHeading>Modification of keys()</p>

<p>List all the constants as an array.</p>

<p class="stepsBox">$keys = array('MODULE_SHIPPING_PERCENT_STATUS', 'MODULE_SHIPPING_PERCENT_RATE');</p>

<p class=contentHeading>Language files</p>

<p>Let's start with the english language file. Define all text constants. They are used for display purpose and are not store in the database.</p>

<p class="noteBox">Note: no text must appear in the program file for localization reasons.</p>

<pre class="stepsBox">
define('MODULE_SHIPPING_PERCENT_TEXT_TITLE', 'Percent Rate');
define('MODULE_SHIPPING_PERCENT_TEXT_DESCRIPTION', 'Percent Rate');
define('MODULE_SHIPPING_PERCENT_TEXT_WAY', 'Best Way'); 
</pre>

<p>Finally, copy the language file to the other language directories.</p>

<p class="stepsBox">copy includes/modules/languages/english/modules/shipping/flat.php to:<br>
includes/modules/languages/espanol/modules/shipping/flat.php and,<br>
includes/modules/languages/german/modules/shipping/flat.php</p>

<p class="content">Translate them (or ask for help if you can't.)</p>

<p class="contentHeading">Test</p>

<p class="content">Test the module thoroughly for all relevant combinations of weight, zones, number of items, etc.</p>

<p class="content">Here are the final <a href="examples/modules/percent.phps">program file</a> and <a href="examples/languages/percent.phps">language file</a>.</p>

<p class="content">Note: do not forget to disable then re-enable the module each time you change the constants.</p>

<p class="content">When everything is polished, send your module as a <a href="http://www.oscommerce.com/downloads.php/contributions/action,contribute">contribution</a>. This will be greatly appreciated.</p>

<p class="content" align="right">Christian Lescuyer&nbsp;- <a href="mailto:tep@limproviste.com">tep@limproviste.com</a>&nbsp;</p>
