<?php
/*
  $Id: shipping_modules_structure.php,v 1.1 2002/02/02 20:32:09 dgw_ Exp $

  osCommerce, Open Source E-Commerce Solutions
  http://www.oscommerce.com

  Copyright (c) 2002 osCommerce

  Released under the GNU General Public License
*/
?>
<p class=contentHeading>General</p>

<p>Shipping modules are implemented as PHP classes. The class name is the same as the module name.</p>

<p class="noteBox"> Class &#147;flat&#148; is described in file flat.php.</p>

<p>As for all classes, you get a constructor function with the same name as the class. This function is called when you create an object belonging to the class. It takes care of initialization tasks. In our case, it sets up necessary variables.</p>

<p class="noteBox">Function flat() is the constructor for class &#147;flat.&#148;</p>

<p>Then you get a number of functions to take care of the various tasks needed for a shipping module.</p>

<ul>
  <li><b>selection()</b> function outputs the HTML for shipping method validation in checkout_address;</li>
  <li><b>quote()</b> is involved with the calculation of the shipping costs;</li>
  <li><b>cheapest()</b> determines which shipping method is the cheapest, for preselection purpose;</li>
  <li><b>display()</b> function outputs the HTML for shipping selection in checkout_payment;</li>
  <li><b>confirm()</b> will return the selected cost and method for confirmation;</li>
  <li><b>check()</b> tells if the module is active or not;</li>
  <li><b>install()</b> is used at installation type to insert the keys in the database;</li>
  <li><b>remove()</b> is used to delete the keys from the database;</li>
  <li><b>keys()</b> returns the key list.</li>
</ul>

<p>Let's have a look at the internals of these functions.</p>

<p>Note: To learn more about PHP classes, <a href="http://www.php.net/manual/en/language.oop.php">see here</a>.</p>

<p class=contentHeading>selection()</p>

<p>Outputs a table with the shipping method title and a checkbox. Each user can select which shipping methods to display among the methods enabled by the administrator.</p>

<p class="noteBox">French user selects &#147;Fedex&#148; and disables USPS shipping.</p>

<p>Constant MODULE_SHIPPING_xxx_TEXT_TITLE is used for the method title.</p>

<p class="noteBox">The value of MODULE_SHIPPING_FLAT_TEXT_TITLE is 'flat.'</p>

<p class=contentHeading>quote()</p>

<p>This is where the cost calculation takes place. The function can be very simple or as complex as need be. Calculations are based on information on the order available as global variables. These variables include:</p>

<ul>
  <li><b>$address_values</b> This is an associative array. Keys include 'country_id' and 'postcode.'</li>
  <li><b>$total_weight</b> Total weight of the order.</li>
  <li><b>$shipping_num_boxes</b> If total weight is greater than SHIPPING_MAX_WEIGHT, the weight is split in a number of boxes weighing only $shipping_weight each. This assumes the load can be split evenly.</li>
  <li><b>$shipping_weight</b> Might be total weight or split weight, see above.</li>
  <li><b>$total_count</b> Number of items in the cart.</li>
  <li><b>$cart-&gt;show_total()</b> Get the total products price.</li>
  <li><b>$cart-&gt;show_products()</b> Get an array with all products details. That would cover extremely complex shipping.</li>
</ul>

<p class="stepsBox">$destination = tep_get_countries($address_values['country_id'], '1');</p>

<p class="stepsBox">$shipping_item_cost = SHIPPING_HANDLING + (MODULE_SHIPPING_ITEM_COST * $total_count);</p>

<p class="stepsBox">$shipping_item_cost = SHIPPING_HANDLING + (MODULE_SHIPPING_PERCENT_COST * $cart-&gt;show_total());</p>

<p class="noteBox">Note: values are specified in the default unit. SHIPPING_HANDLING=3 means $3.00 or 3,00&nbsp;&euro;. Shipping weight might be in pounds or kilos. Be very careful.</p>

<p>Results of the calculations are made available to the rest of the software as global variables. These variables include:</p>

<ul>
  <li><b>$shipping_quoted</b> Shipping method name.</li>
  <li><b>$shipping_xxx_cost</b> Cost of shipping.</li>
  <li><b>$address_xxx_method</b> Precision about the shipping method, or error message.</li>
</ul>

<p class="stepsBox">$shipping_item_method = MODULE_SHIPPING_ITEM_TEXT_WAY;</p>

<p class="stepsBox">if ( $shipping == -1) { $shipping_zones_method = MODULE_SHIPPING_ZONES_UNDEFINED_RATE; }</p>

<p class=contentHeading>cheapest()</p>

<p>This function will determine if this module provides the lowest shipping cost. If it does, it will set global variables.</p>

<ul>
  <li><b>$shipping_cheapest</b> Set to this method's name if cheapest.</li>
  <li><b>$shipping_cheapest_cost</b> Set to this method's cost if cheapest.</li>
</ul>

<p class=contentHeading>display()</p>

<p>The HTML output for shipping method selection is done here. It is possible do display an icon in front of the method name.</p>

<p>The selection button will be displayed only if there are more than one method to choose from.</p>

<p class=contentHeading>confirm()</p>

<p>This function is called at checkout confirmation. Only the selected shipping method will set the global variables.</p>

<ul>
  <li><b>$shipping_cost</b> Cost for the selected method.</li>
  <li><b>$shipping_method</b> Selected method.</li>
</ul>

<p class=contentHeading>check()</p>

<p>Returns the shipping module status as 'true' or 'false.'</p>

<p class=contentHeading>install()</p>

<p>This function will insert the module keys in the database. Keys go in TABLE_CONFIGURATION with a title, value, description, group_id, sort_order and date_added.</p>

<ul>
  <li><b>configuration_title</b> 'Enable Item Shipping'</li>
  <li><b>configuration_key</b> 'MODULE_SHIPPING_ITEM_STATUS'</li>
  <li><b>configuration_value</b> '1'</li>
  <li><b>configuration_description</b> 'Do you want to offer per item rate shipping?'</li>
  <li><b>configuration_group_id</b> '6'</li>
  <li><b>sort_order</b> '7',</li>
  <li><b>date_added</b> now()</li>
</ul>

<p>Example 2</p>

<ul>
  <li><b>configuration_title</b> 'Per Item shipping cost'</li>
  <li><b>configuration_key</b> 'MODULE_SHIPPING_ITEM_COST'</li>
  <li><b>configuration_value</b> '2.50'</li>
  <li><b>configuration_description</b> 'How much will be charged for each item ordered?'</li>
  <li><b>configuration_group_id</b> '6'</li>
  <li><b>sort_order</b> '8'</li>
  <li><b>date_added</b> now()</li>
</ul>

<p>The administrator can change values at a later time from admin.</p>

<p>This is triggered from admin by clicking on the faded green dot in modules/shipping.</p>

<p class=contentHeading>remove()</p>

<p>Removes the keys from the database. This is triggered from admin by clicking on the faded red dot in modules/shipping. See install().</p>

<p class="noteBox">Note: changed values are lost. If the administrator customized values, they must be written down somewhere before disabling the module.</p>

<p class=contentHeading>keys()</p>

<p>Returns the keys from the database as an array.</p>

<p class="content" align="right">Christian Lescuyer - <a href="mailto:tep@limproviste.com">tep@limproviste.com</a></p>
